
require 'fileutils'
require 'myshell'

class ActionResult
  attr_accessor :errors, :output, :success
end


class Target
  include FileUtils
  include MyShell

  attr_reader :revision, :config, :session, :item, :dep_items

  def initialize(revision, dep_items, config, session)
    @revision = revision
    @dep_items = dep_items
    @config = config
    @session = session
    @item = nil
  end

  def host
    return session.host
  end

  def time
    return session.time
  end

  def path
    return revision.checkout_path
  end

  def deps_have_changed?
    return session.has_any?(revision.deps)
  end

  def deps_are_successful?
    return dep_items.inject(true) do |cur, item|
      cur && item.success
    end
  end


  def to_s
    return revision.to_s
  end

  def obtain_item(store)
    @item = store.item_for_target(self, @dep_items)
  end

  def save(store)
    store.save(@item)
  end

  def set_progress(bool)
    @item.set_progress(bool)
  end

  def fire
    context = Context.new(config, @item)
    cd path do 
      execute_actions(context)
    end
    @session.add(@revision, @item)
  end

  def execute_actions(context)
    @item.set_success(false)
    context.actions do |action|
      if not execute_action(action).success then
        return
      end
    end
    @item.set_success(true)
  end
  
  def execute_action(action)
    # TODO: use config.environment here.
    result = action_result(action)
    @item.add_action_result(action, result)
    return result
  end

  def action_result(command)
    result = ActionResult.new  
    begin
      output, errors = execute(command)
      result.success = true
    rescue ShellFailure => e
      output = e.output
      errors = e.errors
      result.success = false
    end
    # TODO update database to accomodate separate errors field
    result.output = output + errors
    #result.errors = errors
    return result
  end

end



class TargetFactory

  def initialize(session, config)
    @session = session
    @config = config
  end

  def target(revision, dep_items)
    return Target.new(revision, dep_items, @config, @session)
  end

end
