
require 'component'

class Visitor
  def initialize(target_factory, revision_factory, roots, builder, log)
    @target_factory = target_factory
    @revision_factory = revision_factory
    @roots = roots
    @builder = builder
    @log = log    
    @visited = {}
  end

  def build_roots
    @roots.each do |root|
      build(root)
    end
  end

  def build(name)
    component = Component.new(name)
    build_component(component)
  end

  def build_component(component)
    if not @visited.has_key?(component) then
      @log.info("component #{component} may require a build.")
      @visited[component] = build_revision_for_component(component)
    end
    return @visited[component]
  end
  
  def build_revision_for_component(component)
    revision = @revision_factory.revision(component)
    if revision.virtual? then
      @log.info("revision #{revision} is virtual; no build performed.")
      return nil
    end
    return build_revision(revision)
  end

  def build_revision(revision)
    dep_items = build_deps(revision.deps)
    return build_revision_with_deps(revision, dep_items)
  end

  def build_revision_with_deps(revision, dep_items)
    target = @target_factory.target(revision, dep_items)
    item = @builder.build(target)
    return item
  end

  def build_deps(deps)
    result = []
    @log.indented do
      result = deps.collect do |dep|
        build_component(dep)
      end
    end
    return result.compact
  end

end



